Package: CaPO4Sim
Type: Package
Title: A Virtual Patient Simulator in the Context of Calcium and
        Phosphate Homeostasis
Version: 0.2.1
Authors@R: c(
    person("David", "Granjon", role = c("aut", "cre", "cph"), email = "dgranjon@ymail.com"),
    person("Diane", "de Zélicourt", role = "cph"),
    person("Vartan", "Kurtcuoglu", role = "cph"),
    person("Olivier", "Bonny", role = "cph"),
    person("François", "Verrey", role = "cph"),
    person(family = "University of Lausanne", role = "fnd"),
    person(family = "University of Zurich", role = "fnd"),
    person(family = "Kidney NCCR.CH", role = "fnd"),
    person(family = "The Interface Group", role = "cph", comment = "Hosting Group"),
    person(family = "RinteRface", role = "cph", comment = "R/HTML Templates")
    )
Maintainer: David Granjon <dgranjon@ymail.com>
Description: Explore calcium (Ca) and phosphate (Pi) homeostasis with two novel 'Shiny' apps, 
    building upon on a previously published mathematical model written in C, 
    to ensure efficient computations. The underlying model is accessible
    here <https://pubmed.ncbi.nlm.nih.gov/28747359/)>.
    The first application explores the fundamentals of Ca-Pi homeostasis, 
    while the second provides interactive case studies for in-depth exploration of the topic, 
    thereby seeking to foster student engagement and an integrative understanding of Ca-Pi regulation.
Imports: shiny, shinyjs, shinyWidgets, shinyjqui, plotly, rintrojs,
        shinycssloaders, visNetwork, purrr, DT, magrittr, utils
Suggests: htmltools, shinydashboard, shinydashboardPlus
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2024-08-18 13:30:12 UTC; davidgranjon
Author: David Granjon [aut, cre, cph],
  Diane de Zélicourt [cph],
  Vartan Kurtcuoglu [cph],
  Olivier Bonny [cph],
  François Verrey [cph],
  University of Lausanne [fnd],
  University of Zurich [fnd],
  Kidney NCCR.CH [fnd],
  The Interface Group [cph] (Hosting Group),
  RinteRface [cph] (R/HTML Templates)
Repository: CRAN
Date/Publication: 2024-08-18 13:50:02 UTC
Built: R 4.6.0; ; 2026-04-30 08:33:54 UTC; unix
