generate_questions <- function() {
  questions <- c(
    "Q1: Summarize all abnormalities",
    "Q2: Does this explain all of the patient's symptoms?",
    "Q3: Why does the patient have a renal insufficiency?",
    "Q4: Establish a differential diagnostic",
    "Q5: What additional lab test would you perform?",
    "Q6: Comment on the 25-(OH)- and 1,25(OH)2-vitamin D ratio",
    "Q7: Why is the PTH undetectable?",
    "Q8: Pose your diagnostic"
  )
  return(questions)
}